// Copyright (C) 1999-2002, Matevz Tadel.
// This file is part of fltk-mt. License: GNU LGPL version 2.

#include <FL/Fl_Widget_Labeler.H>
#include <FL/Fl_Widget.H>

/************************************************************************/
// protected
/************************************************************************/

void
Fl_Widget_Labeler::new_chunk() {
  _chunks.push_back( new char[_chunk_size] );
  _cur_chunk_pos = _chunks.back();
  _cur_chunk_end = _cur_chunk_pos + _chunk_size;
}

/************************************************************************/
// public
/************************************************************************/

Fl_Widget_Labeler::Fl_Widget_Labeler(int s) : _chunk_size(s)
{
  _cur_chunk_end = _cur_chunk_pos = 0;
}

Fl_Widget_Labeler::~Fl_Widget_Labeler()
{
  for(std::list<char*>::iterator i=_chunks.begin(); i!=_chunks.end(); ++i)
    delete [] *i;
}

void
Fl_Widget_Labeler::label_widget(Fl_Widget* w, const char* l)
{
  int strl  = strlen(l) + 1;
  int n = (strl > _chunk_size) ? _chunk_size : strl;
  if(n >= _cur_chunk_end - _cur_chunk_pos) new_chunk();
  strncpy(_cur_chunk_pos, l, n);
  if(strl > _chunk_size) _cur_chunk_pos[n-1] = 0;
  w->label(_cur_chunk_pos);
  _cur_chunk_pos += n;
}

void
Fl_Widget_Labeler::labelf_widget(Fl_Widget* w, const char* fmt, ...)
{
  char buf[_chunk_size];

  va_list ap;
  va_start(ap,fmt);
  int n = vsnprintf(buf, _chunk_size, fmt, ap);
  va_end(ap);
  if(n >= _chunk_size)	n = _chunk_size;
  else			++n;

  if(n >= _cur_chunk_end - _cur_chunk_pos)
    new_chunk();
  strncpy(_cur_chunk_pos, buf, n);
  w->label(_cur_chunk_pos);
  _cur_chunk_pos += n;
}
