#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_OutputPack.H>
#include <FL/Fl_SWM.H>
#include <FL/Fl_RGBA_Button.H>

#include <stdio.h>

struct OP : public Fl_Window, public Fl_SWM_Client
{
  Fl_Value_Input* max_in;
  Fl_OutputPack*  out_pack;

  OP() : Fl_Window(80, 40, "OutputPack Demo:"), Fl_SWM_Client()
  {
    Fl_Button* b = new Fl_Button(0,0,2,1,"@3>>"); b->labeltype(FL_SYMBOL_LABEL);
    set_swm_hotspot_cb(b);

    max_in = new Fl_Value_Input(2,0,4,1);
    max_in->range(10,1000); max_in->step(1);
    max_in->callback((Fl_Callback*) s_set_max, this);

    b = new Fl_Button(6,0,10,1,"Insert 10");
    b->callback((Fl_Callback*) s_insert, this);

    Fl_RGBA_Button *rb = new Fl_RGBA_Button(16,0,8,1,"Bg Col");
    rb->on_change((Fl_Callback*) s_bg_col, this);
    
    out_pack = new Fl_OutputPack(0,1,80,39);

    end();
    resizable(out_pack);

    max_in->value(60);
    max_in->do_callback();

    out_pack->bg_color((Fl_Color)0x20100000);
    out_pack->fg_color((Fl_Color)0xdfefff00);
  }

  static void s_set_max(Fl_Value_Input *w, OP* o)
  { o->out_pack->max_lines((int)(w->value())); }

  static void s_insert(Fl_Widget *w, OP* o) { o->insert(); }
  void insert() {
    char buf[256];
    static int val = 1;
    for(int i=1; i<=10; ++i) {
      sprintf(buf, "Entry %d, local %d", val++, i);
      out_pack->add_line(buf);
    }
  }

  static void s_bg_col(Fl_RGBA_Button *w, OP* o)
  { o->out_pack->bg_color(w->get_color()); }

};

main()
{
  Fl_SWM_Manager mm;
  OP op;
  mm.adopt_window(&op);
  op.show();
  Fl::run();
}
