#include <FL/Fl_ScrollPack.H>
#include <FL/Fl_SWM.H>
#include <FL/Fl_Widget_Labeler.H>
#include <FL/Fl.H>

#include <FL/Fl_Pack.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>

#include <iostream>

#include <unistd.h>
#include <pwd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

int s_defW = 80;
int s_defH = 30;

class TTSP_Test : public Fl_Window,
                  public Fl_Widget_Labeler,
		  public Fl_SWM_Client
{
  Fl_Input* find_cmd;
  Fl_Input* run_cmd;

  Fl_ScrollPack* canvas;
public:
  TTSP_Test() : Fl_Window(s_defW, s_defH, "TTSP_Test")
  {
    swm_size_range = new SWM_Size_Range(s_defW, 8, s_defW, 100);
    //swm_size_range = new SWM_Size_Range(40, 8, 120, 100);

    Fl_Button* b = new Fl_Button(0, 0, 10, 1, "find");
    b->callback((Fl_Callback*) find_cb, this);
    find_cmd = new Fl_Input(10,0,50,1);
    find_cmd->value("find .. -name \\*.H -or -name \\*.cxx");
    run_cmd  = new Fl_Input(60,0,15,1);
    run_cmd->value("gview");
    b = new Fl_Button(75, 0, 5, 1, "clear");
    b->callback((Fl_Callback*) clear_cb, this);

    b = new Fl_Button(78, 1, 2, 1, "@3>>"); b->labeltype(FL_SYMBOL_LABEL);
    set_swm_hotspot_cb(b);

    canvas = new Fl_ScrollPack(0, 2, s_defW, s_defH-1);
    canvas->end();

    resizable(canvas);
    end();
  }

  void find() {
    static struct stat fs;

    FILE* f = popen(find_cmd->value(), "r");
    char file[1024];
    while(!feof(f) && fgets(file, 1024, f)) {
      { char * eol = index(file, 10); if(eol) *eol=0; }
      stat(file, &fs);

      Fl_Pack* h_pack = new Fl_Pack(0,0, s_defW-2, 1);
      h_pack->type(FL_HORIZONTAL);

      Fl_Button* b = new Fl_Button(0, 0, 40, 1);
      label_widget(b, file);
      b->align(FL_ALIGN_INSIDE | FL_ALIGN_LEFT);
      b->callback((Fl_Callback*) exec_cb, this);

      Fl_Box* x;
      x = new Fl_Box(FL_FLAT_BOX, 0,0,10,1, 0);
      labelf_widget(x,"%.1f", fs.st_size/1024.0);
      x = new Fl_Box(FL_FLAT_BOX, 0,0,10,1, 0);
      labelf_widget(x,"%o", fs.st_mode);
      struct passwd* ps = getpwuid(fs.st_uid);
      x = new Fl_Box(FL_FLAT_BOX, 0,0,10,1, 0);
      label_widget(x, ps->pw_name);

      h_pack->end();
      swm_manager->prepare_group(h_pack);
      canvas->sp_pack->add(h_pack);
    }
    canvas->sp_scroll->redraw();
    pclose(f);
  }

  void clear() {
    canvas->sp_pack->clear();
    canvas->sp_pack->resize(canvas->sp_scroll->x(), canvas->sp_scroll->y(),
			    canvas->sp_pack->w(), swm_manager->cell_h());
    canvas->sp_scroll->redraw();
  }

  void exec(const char* fn) {
    std::string s(run_cmd->value()); s += " ";
    s += fn; s += " &";
    system(s.c_str());
  }

  static void find_cb(Fl_Widget* w, TTSP_Test* t) { t->find(); }
  static void clear_cb(Fl_Widget* w, TTSP_Test* t) { t->clear(); }
  static void exec_cb(Fl_Button* w, TTSP_Test* t) { t->exec(w->label()); }
};

int
main(int argc, char** argv) {
  Fl_SWM_Manager x;
  //x.Show();
  TTSP_Test ttsp;
  x.adopt_window(&ttsp);
  ttsp.show();
  Fl::run();
}
